#include "crapfile.h"

#ifdef PIXELTOASTER

#include <string.h>

CrapFile::CrapFile(const char *filename, const char *mode)
{
	mCanWrite = mode[0] == 'w';
	if (strstr(filename, "ms0:/") == filename)
	{
		filename += strlen("ms0:/");
	}
	mFile = fopen(filename, mCanWrite ? "wb" : "rb");
}

CrapFile::~CrapFile()
{
	if (mFile) fclose(mFile);
}

bool CrapFile::IsOpen() const
{
	return mFile != 0;
}

void CrapFile::Read(void* buf, int numBytes)
{
	if (!mCanWrite)
	{
		fread(buf, numBytes, 1, mFile);
	}
}

void CrapFile::Write(void* buf, int numBytes)
{
	if (mCanWrite)
	{
		fwrite(buf, numBytes, 1, mFile);
	}
}

#else

#define O_RDONLY    0x0001 
#define O_WRONLY    0x0002 
#define O_RDWR      0x0003 
#define O_NBLOCK    0x0010 
#define O_APPEND    0x0100 
#define O_CREAT     0x0200 
#define O_TRUNC     0x0400 
#define O_NOWAIT    0x8000 

extern "C"
{
	int sceIoOpen(const char* file, int mode); 
	void sceIoClose(int fd); 
	int sceIoRead(int fd, void *data, int size); 
	int sceIoWrite(int fd, void *data, int size); 
	int sceIoLseek(int fd, long long offset, int whence); 
	int sceIoRemove(const char *file); 
	int sceIoMkdir(const char *dir, int mode); 
	int sceIoRmdir(const char *dir); 
	int sceIoRename(const char *oldname, const char *newname); 
}

CrapFile::CrapFile(const char *filename, const char *mode)
{
	int m = mode[0] == 'w' ? O_WRONLY : O_RDONLY;
	mFD = sceIoOpen(filename, m);
}

CrapFile::~CrapFile()
{
	if (mFD) sceIoClose(mFD);
}

bool CrapFile::IsOpen() const
{
	return mFD != 0;
}

void CrapFile::Read(void* buf, int numBytes)
{
	sceIoRead(mFD, buf, numBytes);
}

void CrapFile::Write(void* buf, int numBytes)
{
	sceIoWrite(mFD, buf, numBytes);
}

#endif
