#include "crapgfx.h"
#include "puff.h"
#include "print.h"

// thanks to nem's helloworld for these (and startup.s...)
#ifdef PIXELTOASTER
extern unsigned short* gPTVRAM;
#define VRAM gPTVRAM
#else
#define VRAM ((unsigned short *)0x4000000)
#endif
#define	FRAMESIZE 0x22000
#define	LINESIZE 512
extern "C"
{
	void pspDisplaySetMode(long,long,long);
	void pspDisplaySetFrameBuf(unsigned short *topaddr,long linesize,long pixelsize,long);
}


// -------------------------
CrapGfx::CrapGfx(const CrapFont& font) : mFont(font), mCurFrame(0)
{
	pspDisplaySetMode(0, SCREEN_WIDTH, SCREEN_HEIGHT);
	unsigned short *p = GetVramAddr(0, 0);
	for (int i = 0; i < FRAMESIZE*2; ++i) *p++ = 0;
	pspDisplaySetFrameBuf(VRAM + mCurFrame * FRAMESIZE, LINESIZE, 1, 1);
}

// -------------------------
unsigned short *CrapGfx::GetVramAddr(int x, int y, bool uncached)
{
	unsigned short* ret = VRAM + (mCurFrame * FRAMESIZE) + (y * LINESIZE) + x;
#ifdef PIXELTOASTER
	return ret;
#else
	return (unsigned short *)((unsigned int)ret | (uncached ? 0x40000000 : 0));
#endif
}

// -------------------------
CrapGfx::~CrapGfx()
{
}

// -------------------------
void CrapGfx::DrawLetter(unsigned short *vram, unsigned char *src)
{
	for (int i = 0; i < mFont.charHeight; ++i)
	{
		for (int j = 0; j < mFont.charWidth; ++j)
		{
			if (*src) *vram++ = 0xffff;
			else *vram++ = 0x0000;
			++src;
		}
		src += mFont.bitmapWidth - mFont.charWidth;
		vram += LINESIZE - mFont.charWidth;
	}
}

// -------------------------
void CrapGfx::Fill(unsigned short colour)
{
	unsigned short *dest = GetVramAddr(0, 0);
	for (int i = 0; i < FRAMESIZE; ++i)
	{
		*dest++ = colour;
	}
}

static bool IsAccent(char c)
{
	return (c == '1' ||
			c == '2' ||
			c == '3' ||
			c == '4' ||
			c == '5' ||
			c == '8');
}


// -------------------------
int CrapGfx::DrawString(int x, int y, const char *str, bool pinyinAccents)
{
	int origX = x;
	unsigned short *dest = GetVramAddr(x, y);
	const char *cur = str;
	int lastVowelX = -1;
	int lastVowelY = -1;
	char lastVowelLetter = 127;
	int lastCharacterX = -1;
	int lastCharacterY = -1;
	int i;

	while (*cur)
	{
		if (*cur == ' ')
		{
			x += mFont.charWidth;
			lastVowelLetter = 127;
			lastVowelX = -1;
		}
		else if (*cur == '\n')
		{
			x = origX;
			y += mFont.charHeight + mFont.charHeight / 4;
			lastVowelLetter = 127;
			lastVowelX = -1;
		}
		else if (*cur >= '!' && *cur <= '~')
		{
			if (pinyinAccents && IsAccent(*cur))
			{
				if (*cur == '1')
				{
					for (i = 0; i < mFont.charWidth; ++i)
						DrawPixel(lastVowelX + i, lastVowelY - 2, 0xffff);
				}
				else if (*cur == '2')
				{
					for (i = 0; i < mFont.charWidth; ++i)
						DrawPixel(lastVowelX + i, lastVowelY - i/2, 0xffff);
				}
				else if (*cur == '3')
				{
					for (i = 0; i < mFont.charWidth/2; ++i)
						DrawPixel(lastVowelX + i, lastVowelY - mFont.charWidth/2 + i, 0xffff);
					for (; i < mFont.charWidth+1; ++i)
						DrawPixel(lastVowelX + i, lastVowelY + mFont.charWidth/2 - i, 0xffff);
				}
				else if (*cur == '4')
				{
					for (i = 0; i < mFont.charWidth; ++i)
						DrawPixel(lastVowelX + i, lastVowelY - mFont.charWidth/2 + i/2, 0xffff);
				}
				else if (*cur == '5')
				{
				}
				else if (*cur == '8')
				{
					DrawPixel(lastCharacterX + 1, lastCharacterY, 0xffff);
					DrawPixel(lastCharacterX + 2, lastCharacterY, 0xffff);
					DrawPixel(lastCharacterX + mFont.charWidth - 2, lastCharacterY, 0xffff);
					DrawPixel(lastCharacterX + mFont.charWidth - 3, lastCharacterY, 0xffff);
				}
				else
				{
					Assert(false && "accent type not handled yet");
				}
			}
			else
			{
				lastCharacterX = x;
				lastCharacterY = y;
				DrawLetter(GetVramAddr(x, y), &mFont.data[(*cur - '!') * mFont.charWidth]);

				if (pinyinAccents)
				{
					switch (*cur)
					{
						case 'a': case 'A':
						case 'e': case 'E':
						case 'i': case 'I':
						case 'o': case 'O':
						case 'u': case 'U':
							if (lastVowelX == -1 && lastVowelLetter >= *cur)
							{
								lastVowelX = x;
								lastVowelY = y;
								lastVowelLetter = *cur;
							}
							break;
					}
				}

				x += mFont.charWidth;
			}
		}
		++cur;
	}

	return x;
}

int CrapGfx::DrawStringf(int x, int y, const char *fmt, ...)
{
	va_list args;
	char buf[4096];
	int i;

	va_start(args, fmt);
	i = dvsprintf(buf, fmt, args);
	va_end(args);

	return DrawString(x, y, buf);
}

// -------------------------
void CrapGfx::DrawGlyph(int x, int y, unsigned char *decompressed)
{
	unsigned short *vram = GetVramAddr(x, y);

	for (int i = 0; i < GLYPH_SIZE; ++i)
	{
		for (int j = 0; j < GLYPH_SIZE; ++j)
		{
			unsigned short level = (*decompressed++) >> 3;
			unsigned short colour = (level << 10) | (level << 5) | level;
			*vram++ = 0x8000 | (colour);
		}
		vram += LINESIZE - GLYPH_SIZE;
	}
}

// -------------------------
void CrapGfx::Flip()
{
	pspDisplaySetFrameBuf(VRAM + mCurFrame * FRAMESIZE, LINESIZE, 1, 1);
	mCurFrame ^= 1;
}

// -------------------------
static unsigned short sImageDecompressBuffer[480*272];
void CrapGfx::DrawCompressedImage(int x, int y, const unsigned char *compressedImageData, unsigned int width, unsigned int height)
{
	Assert(width <= 480 && "width larger than expected");
	Assert(height <= 272 && "height larger than expected");
	unsigned long decompsize = sizeof(sImageDecompressBuffer);
	unsigned long compsize = 0xffffffff;
	int rval = puff((unsigned char *)sImageDecompressBuffer, &decompsize, compressedImageData, &compsize);
	Assert(rval == 0);

	unsigned short *dest = GetVramAddr(x, y);
	unsigned short *src = sImageDecompressBuffer;
	for (unsigned int j = y; j < y + height; ++j)
	{
		for (unsigned int i = x; i < x + width; ++i)
		{
			*dest++ = *src++;
		}
		dest += LINESIZE - width;
	}
}

// -------------------------
void CrapGfx::DrawRect(int x0, int y0, int x1, int y1, unsigned short colour)
{
	unsigned short *topleft = GetVramAddr(x0, y0);
	unsigned short *bottomleft = GetVramAddr(x0, y1);
	for (int x = x0; x <= x1; x++)
	{
		*topleft++ = colour;
		*bottomleft++ = colour;
	}
	topleft = GetVramAddr(x0, y0);
	unsigned short* topright = GetVramAddr(x1, y0);
	for (int y = y0; y <= y1; y++)
	{
		*topleft = colour;
		*topright = colour;
		topleft += LINESIZE;
		topright += LINESIZE;
	}
}

void CrapGfx::DrawPixel(int x, int y, unsigned short colour)
{
	*GetVramAddr(x, y) = colour;
}
