#ifndef INCLUDED_crapgfx_H
#define INCLUDED_crapgfx_H

#include "base.h"

struct CrapFont
{
	unsigned char *data;
	unsigned int len;
	int charWidth;
	int charHeight;
	int numChars;
	int bitmapWidth;
	CrapFont(unsigned char *data, unsigned int len, int charWidth, int charHeight, int numChars) :
		data(data), len(len), charWidth(charWidth), charHeight(charHeight), numChars(numChars), bitmapWidth(charWidth*numChars)
	{
		Assert(charWidth*charHeight*numChars == len);
	}
};

class CrapGfx : public NonCopyable
{
	public:
		CrapGfx(const CrapFont& font);
		~CrapGfx();
		int DrawString(int x, int y, const char *str, bool pinyinAccents = false);
		int DrawStringf(int x, int y, const char *str, ...);
		void Fill(unsigned short colour);
		void DrawGlyph(int x, int y, unsigned char *decompressed);
		void DrawRect(int x0, int y0, int x1, int y1, unsigned short colour);
		void DrawPixel(int x, int y, unsigned short colour);
		void DrawCompressedImage(int x, int y, const unsigned char *compressedImageData, unsigned int width, unsigned int height);
		void Flip();

		enum { SCREEN_WIDTH = 480, SCREEN_HEIGHT = 272 };

	private:
		CrapFont mFont;
		int mCurFrame;

		unsigned short *GetVramAddr(int x, int y, bool uncached = true);
		void DrawLetter(unsigned short *vram, unsigned char *src);
};

#define GLYPH_SIZE 64


#endif
