#ifndef INCLUDED_crapmenu_H
#define INCLUDED_crapmenu_H

#include "base.h"
#include "crapgfx.h"
#include "crappad.h"
#include "crapsound.h"

enum CrapMenuItemType
{
	Invalid = -1,
	IntSlider,
	BoolToggle,
};

struct CrapMenuItem
{
	const char* prompt;
	CrapMenuItemType type;
	void* value;
	void* aux;
	void (*callback)(CrapMenuItem*, int numItems, int itemChanged);
	CrapMenuItem(const char* prompt, CrapMenuItemType type, void* value, void* aux = 0, void (*callback)(CrapMenuItem *, int numItems, int itemChanged) = 0) : prompt(prompt), type(type), value(value), aux(aux), callback(callback) {}
	CrapMenuItem() : prompt(0), type(Invalid), value(0) {}
};

class CrapMenu : public NonCopyable
{
	public:
		CrapMenu(CrapGfx& gfx, CrapPad& pad, CrapSound& sound, const CrapSoundSample& updown, const CrapSoundSample& change)
			: mGfx(gfx), mPad(pad), mSound(sound), mUpDown(updown), mChange(change), mNumItems(0), mPrompt(0), mConfirmExit(0) {}
		void Add(const CrapMenuItem& item);
		void Go();
		void SetPrompt(const char *prompt) { mPrompt = prompt; }
		void SetExitConfirm(bool (*callback)(CrapMenuItem*, int numItems)) { mConfirmExit = callback; }
	private:
		CrapGfx& mGfx;
		CrapPad& mPad;
		CrapSound& mSound;
		const CrapSoundSample& mUpDown;
		const CrapSoundSample& mChange;
		const char *mPrompt;
		enum { MAX_ITEMS = 20 };
		// can't use any C++ features on psp toolchain yet... (new, virt base, etc.)
		CrapMenuItem mItems[MAX_ITEMS];
		int mNumItems;
		bool (*mConfirmExit)(CrapMenuItem*, int numItems);
};

#endif
