#include "crappad.h"

// from pspchess's controller.h
typedef struct _ctrl_data 
{ 
	unsigned long frame; 			// (tick)
	unsigned long buttons;
	unsigned char analog[4]; 		// [0]:X [1]:Y
	unsigned long unused; 
} ctrl_data_t; 

extern "C"
{
	void  sceCtrlInit(int unknown); 
	void  sceCtrlSetAnalogMode(int on); 
	void  sceCtrlRead(ctrl_data_t* paddata, int unknown); 
}

// ----------------------------
CrapPad::CrapPad() : mBeforeInput(0), mBeforeTick(0)
{
	sceCtrlInit(0);
	sceCtrlSetAnalogMode(0);
}

// ----------------------------
CrapPad::~CrapPad()
{
}

// ----------------------------
unsigned long CrapPad::Read(bool doRepeat)
{
	ctrl_data_t ctl;
	const int REPEAT_TIME = 0x40000;

	sceCtrlRead(&ctl, 1);

	if (ctl.buttons == mBeforeInput)
	{
		if (doRepeat && (ctl.frame - mBeforeTick) > REPEAT_TIME)
		{
			return mBeforeInput;
		}
		return 0;
	}
	mBeforeInput  = ctl.buttons;
	mBeforeTick = ctl.frame;
	return mBeforeInput;
}


int NumBitsSet(unsigned long val)
{
	int n = 0;
	for (; val; val >>= 1) if (val & 1) n++;
	return n;
}

unsigned long CrapPad::WaitForOneOf(int mask, bool doRepeat)
{
	for (;;)
	{
		int ret = Read(doRepeat);
		if ((ret & mask) && NumBitsSet(ret) == 1)
		{
			return ret;
		}
	}
}
