// todo;
// save menu settings

#include "font.inc"
#include "crapgfx.h"
#include "crappad.h"
#include "crapmenu.h"
#include "crapfile.h"
#include "crapsound.h"
#include "glyphs.inc"
#include "puff.h"
#include "rand.h"
#include "print.h"
#include "homebutton.h"
#ifdef DO_AUDIO
#include "move.inc"
#include "intro.inc"
#include "congratlaugh.inc"
#include "error.inc"
#else
const unsigned char AudioData_move[] = { 0 };
const unsigned char AudioData_intro[] = { 0 };
const unsigned char AudioData_congratlaugh[] = { 0 };
const unsigned char AudioData_error[] = { 0 };
#endif
#include "titlescreen.inc"
#include "congrats.inc"

static const EntryData* aCurrentSet[ARRAYSIZE(aEntries)];
int MakeShuffledEntryArray(int beginEntry, int endEntry)
{
	int i;

	int num = endEntry - beginEntry;
	for (i = 0; i < num; ++i) aCurrentSet[i] = 0;

	for (i = beginEntry; i < endEntry; ++i)
	{
		int j = Rand() % num;
		while (aCurrentSet[j] != 0)
		{
			++j;
			if (j == num) j = 0;
		}
		aCurrentSet[j] = &aEntries[i];
	}

	return num;
}

void MakeFirstLessThanSecond(CrapMenuItem* items, int numItems, int changedIndex)
{
	Assert(numItems >= 2);
	if (changedIndex == 0)
	{
		if (*((int*)items[0].value) >= *((int*)items[1].value)) *((int*)items[0].value) = *((int*)items[1].value) - 1;
	}
	else if (changedIndex == 1)
	{
		if (*((int*)items[1].value) <= *((int*)items[0].value)) *((int*)items[1].value) = *((int*)items[0].value) + 1;
	}
}

CrapSound* gCrapSoundExitConfirmHack;

bool ExitConfirm(CrapMenuItem* items, int numItems)
{
	Assert(numItems >= 5);
	if (
		(*((bool*)items[2].value) == false
		&& *((bool*)items[3].value) == false
		&& *((bool*)items[4].value) == false)
		||
		(*((bool*)items[2].value) == true
		&& *((bool*)items[3].value) == true
		&& *((bool*)items[4].value) == true)
		)
	{
		CrapSoundSample error(AudioData_error, sizeof(AudioData_error));
		gCrapSoundExitConfirmHack->Play(error);
		return false;
	}
	return true;
}


struct App
{
	App(CrapGfx& gfx, CrapPad& pad, CrapSound& sound) : mGfx(gfx), mPad(pad), mSound(sound) {}

	void DrawGlyphs(int x, int y, const EntryData* entry)
	{
		unsigned char decomp[GLYPH_SIZE*GLYPH_SIZE];
		int j;

		int numGlyphs = entry->numglyphs;

		for (j = 0; j < numGlyphs; ++j)
		{
			unsigned long decompsize = GLYPH_SIZE*GLYPH_SIZE;
			unsigned long compsize = 0xffffffff;
			int rval = puff(decomp, &decompsize, entry->glyph[j], &compsize);
			if (rval != 0)
			{
				mGfx.DrawString(10, 10, "rval != 0");
				return;
			}

			mGfx.DrawGlyph(x, y, decomp);
			x += GLYPH_SIZE + 4;
		}
	}

	void DoDifficultyChoiceScreen(int& beginEntry, int& endEntry, bool& includeGlyph, bool& includePinyin, bool& includeEnglish)
	{
		{
			CrapFile settings("ms0:/settings.dat", "r");
			if (settings.IsOpen())
			{
				settings.Read(&beginEntry, sizeof(int));
				settings.Read(&endEntry, sizeof(int));
				settings.Read(&includeGlyph, sizeof(bool));
				settings.Read(&includePinyin, sizeof(bool));
				settings.Read(&includeEnglish, sizeof(bool));
			}
		}

		CrapSoundSample updown(AudioData_move, sizeof(AudioData_move));
		CrapSoundSample change(AudioData_move, sizeof(AudioData_move));
		CrapMenu menu(mGfx, mPad, mSound, updown, change);
		int maxInt = ARRAYSIZE(aEntries);
		menu.Add(CrapMenuItem("Beginning entry: %d", IntSlider, &beginEntry, &maxInt, MakeFirstLessThanSecond));
		menu.Add(CrapMenuItem("End entry: %d", IntSlider, &endEntry, &maxInt, MakeFirstLessThanSecond));
		menu.Add(CrapMenuItem("Question includes Glyph: %s", BoolToggle, &includeGlyph));
		menu.Add(CrapMenuItem("Question includes Pinyin: %s", BoolToggle, &includePinyin));
		menu.Add(CrapMenuItem("Question includes English: %s", BoolToggle, &includeEnglish));
		char buf[4096];
		dsprintf(buf,
			"\n\n"
			"There are currently %d entries.\n"
			"They're taken from the New Practical Chinese Reader 1,\n"
			"and are entered in order of appearance in the book.\n\n\n"
			"Up/Down: Choose Item\n"
			"Left/Right: Modify\n"
			"Start: Begin Game", maxInt);
		menu.SetPrompt(buf);
		gCrapSoundExitConfirmHack = &mSound;
		menu.SetExitConfirm(ExitConfirm);
		menu.Go();
		gCrapSoundExitConfirmHack = 0;

		{
			CrapFile settings("ms0:/settings.dat", "w");
			if (settings.IsOpen())
			{
				settings.Write(&beginEntry, sizeof(int));
				settings.Write(&endEntry, sizeof(int));
				settings.Write(&includeGlyph, sizeof(bool));
				settings.Write(&includePinyin, sizeof(bool));
				settings.Write(&includeEnglish, sizeof(bool));
			}
		}
	}

	void DrawEntry(const EntryData* entry, bool glyph, bool pinyin, bool english)
	{
		if (glyph) DrawGlyphs(10, 10, entry);
		if (pinyin) mGfx.DrawString(10, 100, entry->pinyin, true);
		if (english) mGfx.DrawString(10, 150, entry->english);
	}

	void Run()
	{
		int minEntry = 0;
		int maxEntry = 20;
		bool includeGlyph = true;
		bool includePinyin = false;
		bool includeEnglish = false;

		mGfx.DrawCompressedImage(0, 0, CompressedData_titlescreen, Width_titlescreen, Height_titlescreen);
		mGfx.Flip();
		CrapSoundSample intro(AudioData_intro, sizeof(AudioData_intro));
		mSound.Play(intro);
		int dumbCount = 0;
		while (mPad.Read(false) != PAD_START) { ++dumbCount; }
		RandSeed(dumbCount);
		CrapSoundSample sample(AudioData_move, sizeof(AudioData_move));
		mSound.Play(sample);

		while (1)
		{
			DoDifficultyChoiceScreen(minEntry, maxEntry, includeGlyph, includePinyin, includeEnglish);

			int numRemaining = MakeShuffledEntryArray(minEntry, maxEntry);
			int cur = 0;

			while (numRemaining > 0)
			{
				const EntryData* entry = aCurrentSet[cur];

				mGfx.Fill(0);
				DrawEntry(entry, includeGlyph, includePinyin, includeEnglish);
				mGfx.DrawString(10, 255, "X: Answer");
				mGfx.Flip();

				mPad.WaitForOneOf(PAD_CROSS, false);
				
				mGfx.Fill(0);
				DrawEntry(entry, true, true, true);
				mGfx.DrawString(10, 240, "X: Right (Remove)");
				mGfx.DrawString(10, 255, "O: Wrong (Put at back)");
				mGfx.Flip();

				switch (mPad.WaitForOneOf(PAD_CROSS | PAD_CIRCLE, false))
				{
					case PAD_CROSS:
						int i;
						for (i = cur; i < numRemaining - 1; ++i)
						{
							aCurrentSet[i] = aCurrentSet[i + 1];
						}
						--numRemaining;
						if (cur >= numRemaining) cur = 0;
						break;
					case PAD_CIRCLE:
						++cur;
						if (cur >= numRemaining) cur = 0;
						break;
					default:
						Assert(0 && "unexpected button");
						break;
				}
			}

			mGfx.DrawCompressedImage(0, 0, CompressedData_congrats, Width_congrats, Height_congrats);
			mGfx.Flip();
			CrapSoundSample congratlaugh(AudioData_congratlaugh, sizeof(AudioData_congratlaugh));
			mSound.Play(congratlaugh);
			mPad.WaitForOneOf(0xffffffff, false);
		}
	}

private:
	CrapGfx& mGfx;
	CrapPad& mPad;
	CrapSound& mSound;

	App(const App&);
	void operator=(const App&);
};

extern "C" int xmain(void)
{
	CrapFont cf(font_raw, font_raw_len, 8, 13, 94);
	CrapGfx gfx(cf);
	CrapPad pad;
	CrapSound sound;
	SetupHomeButton();
	App app(gfx, pad, sound);
	app.Run();
	return 0;
}
