// Pixel Toaster implementation
// Copyright  Glenn Fiedler
// http://www.pixeltoaster.com

//temporary
#include <cstdio>

#include <cassert>
#include <cstring>

#pragma warning(disable : 4996)

#include "PixelToaster.h"
#include "Common.h"
#include "Conversion.h"

#include "Unix/Index.h"
#include "Apple/Index.h"
#include "Windows/Index.h"

PixelToaster::DisplayInterface* PixelToaster::createDisplay()
{
#ifdef DisplayClass
    return new DisplayClass();
#else
    return 0;       // unsupported platform
#endif
}

PixelToaster::TimerInterface* PixelToaster::createTimer()
{
#ifdef TimerClass
    return new TimerClass();
#else
	return new PortableTimer();
#endif
}

PixelToaster::Converter_PixelToPixel converterPixelToPixel;
PixelToaster::Converter_PixelToXRGB8888 converterPixelToXRGB8888;
PixelToaster::Converter_PixelToXBGR8888 converterPixelToXBGR8888;
PixelToaster::Converter_PixelToRGB888 converterPixelToRGB888;
PixelToaster::Converter_PixelToBGR888 converterPixelToBGR888;
PixelToaster::Converter_PixelToRGB565 converterPixelToRGB565;
PixelToaster::Converter_PixelToBGR565 converterPixelToBGR565;
PixelToaster::Converter_PixelToXRGB1555 converterPixelToXRGB1555;
PixelToaster::Converter_PixelToXBGR1555 converterPixelToXBGR1555;
PixelToaster::Converter_XRGB8888ToPixel converterXRGB8888ToPixel;
PixelToaster::Converter_XRGB8888ToXRGB8888 converterXRGB8888ToXRGB8888;
PixelToaster::Converter_XRGB8888ToXBGR8888 converterXRGB8888ToXBGR8888;
PixelToaster::Converter_XRGB8888ToRGB888 converterXRGB8888ToRGB888;
PixelToaster::Converter_XRGB8888ToBGR888 converterXRGB8888ToBGR888;
PixelToaster::Converter_XRGB8888ToRGB565 converterXRGB8888ToRGB565;
PixelToaster::Converter_XRGB8888ToBGR565 converterXRGB8888ToBGR565;
PixelToaster::Converter_XRGB8888ToXRGB1555 converterXRGB8888ToXRGB1555;
PixelToaster::Converter_XRGB8888ToXBGR1555 converterXRGB8888ToXBGR1555;

PixelToaster::Converter* PixelToaster::requestConverter(PixelToaster::Format source, PixelToaster::Format destination)
{
    if (source==Format::Pixels)
    {
        switch (destination)
        {
            case Format::Pixels: return &converterPixelToPixel;
            case Format::XRGB8888: return &converterPixelToXRGB8888;
            case Format::XBGR8888: return &converterPixelToXBGR8888;
            case Format::RGB888: return &converterPixelToRGB888;
            case Format::BGR888: return &converterPixelToBGR888;
            case Format::RGB565: return &converterPixelToRGB565;
            case Format::BGR565: return &converterPixelToBGR565;
            case Format::XRGB1555: return &converterPixelToXRGB1555;
            case Format::XBGR1555: return &converterPixelToXBGR1555;
			case Format::Unknown: return 0;
        }
    }
    else if (source==Format::XRGB8888)
    {
        switch (destination)
        {
            case Format::Pixels: return &converterXRGB8888ToPixel;
            case Format::XRGB8888: return &converterXRGB8888ToXRGB8888;
            case Format::XBGR8888: return &converterXRGB8888ToXBGR8888;
            case Format::RGB888: return &converterXRGB8888ToRGB888;
            case Format::BGR888: return &converterXRGB8888ToBGR888;
            case Format::RGB565: return &converterXRGB8888ToRGB565;
            case Format::BGR565: return &converterXRGB8888ToBGR565;
            case Format::XRGB1555: return &converterXRGB8888ToXRGB1555;
            case Format::XBGR1555: return &converterXRGB8888ToXBGR1555;
			case Format::Unknown: return 0;
        }
    }

    return 0;
}
