// Windows Display implementation
// Copyright  Glenn Fiedler
// http://www.pixeltoaster.com

class Display : public DisplayAdapter, public Adapter
{
public:

	Display()
	{
		direct3d = Direct3DCreate9(D3D_SDK_VERSION);
		defaults();
	}

	~Display()
	{
		close();

		if (direct3d)
		{
			direct3d->Release();
			direct3d = 0;
		}
	}

	bool open(const char title[], int width, int height, Mode mode, Output output)
	{
		DisplayAdapter::open(title, width, height, mode, output);

		window = new Window(this, title, width, height);

		if (!window->handle())
		{
			close();
			return false;
		}

		if (output==Output::Default)
		{
			#ifndef _DEBUG
			if (fullscreen())
				return true;
			else
				return windowed();
			#else
			return windowed();
			#endif
		}
		else if (output==Output::Fullscreen)
		{
			return fullscreen();
		}
		else
		{
			return windowed();
		}
	}

	void close()
	{
		delete device;
		delete window;

		DisplayAdapter::close();
	}

	bool update(integer32 *integerPixels, Pixel *floatingPointPixels)
	{
		if (window)
			window->update();

		if (device)
			device->update(integerPixels, floatingPointPixels);

		if (window && !window->visible())
		{
			window->show();
			window->update();
		}

		return true;
	}

	void listener(Listener *listener)
	{
		DisplayAdapter::listener(listener);

		if (window)
			window->listener = listener;
	}

	// implement adapter interface for interoperability with window class

	bool paint()
	{
		if (!device)
		{
			HDC dc = GetDC(window->handle());

			if (dc)
			{
				HBRUSH brush = CreateSolidBrush(RGB(0,0,0));
				SelectObject(dc, brush);
				RECT rect;
				GetClientRect(window->handle(), &rect);
				Rectangle(dc, 0, 0, rect.right, rect.bottom);
				DeleteObject(brush);
				ReleaseDC(window->handle(), dc);
			}

			ValidateRect(window->handle(), NULL);

			return true;
		}
		else
		{
			return device->paint();
		}
	}

	bool fullscreen()
	{
		if (device)
		{
			if (output()==Output::Fullscreen)
				return true;

			delete device;
			device = 0;
		}

		window->fullscreen(width(), height());

		device = new Device(direct3d, window->handle(), width(), height(), mode(), false);

		if (!device->valid())
		{
			close();
			return false;
		}

		DisplayAdapter::fullscreen();

		window->update();

		return true;
	}

	bool windowed()
	{
		if (device)
		{
			if (output()==Output::Windowed)
				return true;

			delete device;
			device = 0;
		}

		window->windowed(width(), height());

		device = new Device(direct3d, window->handle(), width(), height(), mode(), true);

		if (!device->valid())
		{
			close();
			return false;
		}

		DisplayAdapter::windowed();

		return true;
	}

	bool toggle()
	{
		if (output()==Output::Windowed)
			return fullscreen();
		else
			return windowed();
	}

protected:

	void defaults()
	{
		DisplayAdapter::defaults();
		window = 0;
		device = 0;
	}

private:

	LPDIRECT3D9 direct3d;
	Window *window;
	Device *device;
};
