#include <windows.h>
#include "PixelToaster.h"
#include "crappad.h"
#include <assert.h>
#include <vector>

extern "C" int xmain();

using namespace PixelToaster;

Display display;

unsigned short *gPTVRAM;
static unsigned long gFrame;

extern "C"
{
    void pspDisplaySetMode(long a,long w, long h)
	{
		assert(a == 0);
		assert(w == 480);
		assert(h == 272);
		int ret = display.open("PSP", 480, 272, PixelToaster::Mode::TrueColor, Output::Windowed);
		assert(ret);
	}

	void pspDisplaySetFrameBuf(unsigned short *topaddr, long linesize, long pixelsize, long waitForVblank)
	{
		assert(linesize == 512);
		assert(pixelsize == 1);
		(void)waitForVblank;
		assert(topaddr == gPTVRAM || topaddr == gPTVRAM + 512*272);
		
		std::vector<unsigned int> pixels(480*272);

		unsigned int index = 0;

		for (int y=0; y<272; y++)
		{
			for (int x=0; x<480; x++)
			{
				unsigned short c = topaddr[y*512+x];
				unsigned int b = ((c & 0x7c00) >> 10) << 3;
				unsigned int g = ((c & 0x3e0) >> 5) << 3;
				unsigned int r = (c & 0x1f) << 3;
				pixels[index] = (r << 16) | (g << 8) | b;
				index++;
			}
		}

        display.update(&pixels[0]);
	}

	void sceCtrlInit(int unknown)
	{
		gFrame = 0;
	}

	void sceCtrlSetAnalogMode(int on)
	{
		(void)on;
	}

typedef struct _ctrl_data 
{ 
	unsigned long frame; 			// (tick)
	unsigned long buttons;
	unsigned char analog[4]; 		// [0]:X [1]:Y
	unsigned long unused; 
} ctrl_data_t; 

	void sceCtrlRead(ctrl_data_t* paddata, int unknown)
	{
		assert(paddata);
		paddata->frame = gFrame++;
		paddata->buttons = 0;
		if (GetAsyncKeyState(VK_UP) & 0x8000) paddata->buttons |= PAD_UP;
		if (GetAsyncKeyState(VK_DOWN) & 0x8000) paddata->buttons |= PAD_DOWN;
		if (GetAsyncKeyState(VK_LEFT) & 0x8000) paddata->buttons |= PAD_LEFT;
		if (GetAsyncKeyState(VK_RIGHT) & 0x8000) paddata->buttons |= PAD_RIGHT;
		if (GetAsyncKeyState('Z') & 0x8000) paddata->buttons |= PAD_CROSS;
		if (GetAsyncKeyState('X') & 0x8000) paddata->buttons |= PAD_CIRCLE;
		if (GetAsyncKeyState('A') & 0x8000) paddata->buttons |= PAD_SQUARE;
		if (GetAsyncKeyState('S') & 0x8000) paddata->buttons |= PAD_TRIANGLE;
		if (GetAsyncKeyState(VK_RETURN) & 0x8000) paddata->buttons |= PAD_START;
	}
}

int __stdcall WinMain(      
    HINSTANCE hInstance,
    HINSTANCE hPrevInstance,
    LPSTR lpCmdLine,
    int nCmdShow
)
{
	gPTVRAM = (unsigned short *)malloc(512*272*2*sizeof(unsigned short));
	xmain();
	free(gPTVRAM);
}
