#include "rand.h"

// this code is taken from the SGADE gameboy library
#define RAND_STATE_VECTOR_LENGTH (624)                 
#define RAND_PERIOD (397)                 
#define RAND_MAGIC (0x9908B0DFU)         
#define RAND_HI_BIT(u) ((u) & 0x80000000U)
#define RAND_LO_BIT(u) ((u) & 0x00000001U)
#define RAND_LO_BITS(u) ((u) & 0x7FFFFFFFU)
#define RAND_MIX_BITS(u, v) (RAND_HI_BIT(u)|RAND_LO_BITS(v))
static unsigned int *s_RandNext;          
static int s_RandLeft = -1;      
static unsigned int s_RandState[ RAND_STATE_VECTOR_LENGTH + 1 ];  

static unsigned int reloadMersenneTwister()
{
	// Variables;
    register unsigned int *p0=s_RandState, *p2=s_RandState+2, *pM=s_RandState+RAND_PERIOD, s0, s1;
    register int j;

	// Code;
    if( s_RandLeft < -1 ) RandSeed( 4357U );

    s_RandLeft = RAND_STATE_VECTOR_LENGTH - 1;
	s_RandNext = s_RandState + 1;

    for( s0 = s_RandState[0], s1 = s_RandState[1], j = RAND_STATE_VECTOR_LENGTH - RAND_PERIOD + 1; --j; s0 = s1, s1 = *p2++ )
	{
        *p0++ = *pM++ ^ (RAND_MIX_BITS(s0, s1) >> 1) ^ (RAND_LO_BIT(s1) ? RAND_MAGIC : 0U);
	}

    for( pM = s_RandState, j = RAND_PERIOD; --j; s0 = s1, s1 = *p2++ )
	{
        *p0++ = *pM++ ^ (RAND_MIX_BITS(s0, s1) >> 1) ^ (RAND_LO_BIT(s1) ? RAND_MAGIC : 0U);
	}

    s1=s_RandState[0], *p0 = *pM ^ (RAND_MIX_BITS(s0, s1) >> 1) ^ (RAND_LO_BIT(s1) ? RAND_MAGIC : 0U);
    s1 ^= (s1 >> 11);
    s1 ^= (s1 <<  7) & 0x9D2C5680U;
    s1 ^= (s1 << 15) & 0xEFC60000U;
    
	return(s1 ^ (s1 >> 18));
}

unsigned int Rand()
{
	unsigned int y;

    if( --s_RandLeft < 0 ) return reloadMersenneTwister();

    y  = *s_RandNext++;
    y ^= (y >> 11);
    y ^= (y <<  7) & 0x9D2C5680U;
    y ^= (y << 15) & 0xEFC60000U;
    
	return(y ^ (y >> 18));
}

void RandSeed(unsigned int a_Seed)
{
	register unsigned int x = (a_Seed | 1U) & 0xFFFFFFFFU, *s = s_RandState;
    register int j;

    for( s_RandLeft = 0, *s++ = x, j = RAND_STATE_VECTOR_LENGTH; 
		 --j; 
		 *s++ = (x*=69069U) & 0xFFFFFFFFU );
}
